/**
 * Disjoint-set
 *
 * Complexity (Time): O(1)
 *   make_set  -> O(1)
 *   find_set  -> O(a(n))
 *   union_set -> O(a(n))
 * Complexity (Space): O(n)
 */

int par[MAX];
int h[MAX];
int size[MAX];

// Initialize element x
void make_set(int x) {
  par[x] = x;
  h[x] = 0;
  size[x] = 1;
}


// Return set index from element x
int find_set(int x) {
  if (par[x] != x)
    par[x] = find_set(par[x]);

  return par[x];
}


// Make x and y belong to the same set
void union_set(int x, int y) {
  int xroot = find_set(x);
  int yroot = find_set(y);

  if (xroot == yroot)
    return;

  if (h[xroot] < h[yroot]) {
    par[xroot] = yroot;
    size[yroot] += size[xroot];
  } else {
    par[yroot] = xroot;
    size[xroot] += size[yroot];

    if (h[xroot] == h[yroot])
      h[xroot]++;
  }
}
