/**
 * Lazy Segment Tree
 *
 * Complexity (Time):
 *   build_tree  -> O(n log n)
 *   update_tree -> O(log n)
 *   query_tree  -> O(log n)
 * Complexity (Space): O(n)
 */

int N, tree[4 * MAX], lazy[4 * MAX], v[MAX];

// Builds tree with elements from v
void build(int node = 1, int a = 0, int b = n) {
  if (a > b) 
    return;

  if (a == b) {
    tree[node] = v[a];
    return;
  }

  build(node * 2, a, (a + b) / 2);
  build(node * 2 + 1, (a + b) / 2 + 1, b);
  tree[node] = tree[node * 2] + tree[node * 2 + 1];
}


// Propagates value to tree and through lazy tree
void push(int node, int a, int b, int val) {
  tree[node] += val;
  // tree[node] += (b - a + 1) * val; (for Range Sum Query)

  if (a != b) {
    lazy[node * 2] += val;
    lazy[node * 2 + 1] += val;
  }

  lazy[node] = 0;
}


// Updates segment [i,j] by adding value val
void update(int i, int j, int val, int node = 1, int a = 0, int b = n) {
  if (lazy[node] != 0)
    push(node, a, b, lazy[node]);

  if (a > b or a > j or b < i) 
    return;

  if (a >= i and b <= j) {
    push(node, a, b, val);
    return;
  }

  update(i, j, val, node * 2, a, (a + b) / 2);
  update(i, j, val, node * 2 + 1, (a + b) / 2 + 1, b);
  tree[node] = tree[node * 2] + tree[node * 2 + 1];
}


// Returns sum of [i,j]
int query(int i, int j, int node = 1, int a = 0, int b = n) {
  if (a > b || a > j || b < i)
    return 0;

  if (lazy[node])
    push(node, a, b, lazy[node]);

  if (a >= i and b <= j)
    return tree[node];

  int q1 = query(i, j, node * 2, a, (a + b) / 2);
  int q2 = query(i, j, node * 2 + 1, (a + b) / 2 + 1, b);
  return q1 + q2;
}
